/* NotificationHelper.cs
 *
 * GNOME Do is the legal property of its developers. Please refer to the
 * COPYRIGHT file distributed with this source distribution.
 *  
 * This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.Drawing;
using System.Windows.Forms;

using Do.Platform;

namespace Do.Platform.Windows {
    public enum NotificationCapability {
        actions,
        append,
        body,
        body_hyperlinks,
        body_images,
        body_markup,
        icon_multi,
        icon_static,
        image_svg,
        max,
        positioning, // not an official capability
        scaling, // not an official capability
        sound
    }

    internal class NotificationHelper {
        private static NotifyIcon m_notifyicon;

        const string DefaultIconName = "gnome-do";

        const int IconSize = 48;
        const int LettersPerWord = 7;
        const int MillisecondsPerWord = 350;
        const int MinNotifyShow = 5000;
        const int MaxNotifyShow = 10000;

        public event EventHandler<NotificationEventArgs> NotificationClosed;

        public NotificationHelper (NotifyIcon icon)
        {
            m_notifyicon = icon;
        }

        static int ReadableDurationForMessage (string title, string message)
        {
            int t = (title.Length + message.Length) / LettersPerWord * MillisecondsPerWord;
            return Math.Min (Math.Max (t, MinNotifyShow), MaxNotifyShow);
        }

        public void Notify (Notification note)
        {
            m_notifyicon.ShowBalloonTip (
                ReadableDurationForMessage (note.Title, note.Body),
                note.Title,
                note.Body,
                ToolTipIcon.Warning);

            m_notifyicon.BalloonTipClosed += (sender, e) => OnNotificationClosed (note);
        }

        public bool SupportsCapability (NotificationCapability capability)
        {
            if (capability == NotificationCapability.body)
                return true;
            return false;
        }

        void OnNotificationClosed (Notification note)
        {
            if (NotificationClosed != null)
                NotificationClosed (this, new NotificationEventArgs (note));
        }
    }
}